﻿/**

Livewire Flash - Core code for running the Livewire algorithm
Copyright (C) 2011  Jeffrey Bush  jeff@coderforlife.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

**/

package Livewire
{
	import flash.events.EventDispatcher;
	import flash.events.ProgressEvent;
	import flash.events.Event;
	
	import flash.utils.setTimeout;
	import flash.utils.clearTimeout;
	//import flash.utils.setInterval;
	//import flash.utils.clearInterval;
	import flash.utils.getTimer;
	
	
	CONFIG::DEBUG
	{
		import com.demonsters.debugger.MonsterDebugger;
	}
	
	public class Threaded extends EventDispatcher
	{
		public static const PROGRESS:String = "workProgress";
		public static const FINISHED:String = "threadFinished";

		private static const runtime:int = Math.floor(1000 / CONFIG::FRAME_RATE) - 1;

		private var _name:String;
		CONFIG::DEBUG
		{
			private var _timer:int;
		}
		private var _isExecuting:Boolean = false;
		private var _timeoutId:uint = 0;
		private var _latency:Number; // cycles/ms
		private var _init_iters:uint;
		private var _progress:int = 0, _partialProgress:int = 0, _inc:int = 1, _totalProgress:int = 100;
		private var _context;

		public final function Threaded(name:String) { this._name = name; }

		public final function GetProgress():int { return this._progress; }
		protected final function SetProgress(value:int):void
		{
			this._partialProgress += value - this._progress;
			this._progress = value;
			if (this._partialProgress >= this._inc)
			{
				this._partialProgress %= this._inc;
				this.dispatchEvent(new ProgressEvent(Threaded.PROGRESS, false, false, this._progress, this._totalProgress));
				CONFIG::DEBUG
				{
					this.Checkpoint(Math.round(100*this._progress/this._totalProgress)+'%');
				}
			}
		}
		public final function GetTotalProgress():int { return this._totalProgress; }
		protected final function SetTotalProgress(value:int):void { this._totalProgress = value; this._inc = this._totalProgress / 100; }
		protected final function IncProgress():void
		{
			++this._progress;
			if (++this._partialProgress == this._inc)
			{
				this._partialProgress = 0;
				this.dispatchEvent(new ProgressEvent(Threaded.PROGRESS, false, false, this._progress, this._totalProgress));
				CONFIG::DEBUG
				{
					this.Checkpoint(Math.round(100*this._progress/this._totalProgress)+'%');
				}
			}
		}

		protected final function Checkpoint(info:String):void
		{
			CONFIG::DEBUG
			{
				trace(this._name+" thread took "+((getTimer() - this._timer)/1000.0)+" secs to get to "+info);
				//MonsterDebugger.trace(this, this._name+" thread took "+((getTimer() - this._timer)/1000.0)+" secs to get to "+info);
			}
		}
		public final function IsExecuting():Boolean { return this._isExecuting; }

		protected final function start(context:*= null):void
		{
			Stop();
			this._isExecuting = true;
			this._progress = 0;
			this._context = context == null ? {} : context;
			setTimeout(this.RunFirst, 1);
		}
		public final function Stop():void
		{
			if (this._timeoutId != 0)
			{
				clearTimeout(this._timeoutId);
				this._timeoutId = 0;
			}
			if (this._isExecuting)
			{
				this._isExecuting = false;
				this.RunLast();
			}
		}

		protected function StepFirst(context:*):void {}	// always run first
		// run until returns false or Stop() is called, performs many iterations
		protected function Step(iters:uint, context:*):Boolean { return false; }
		protected function StepLast(context:*):void {}		// always run last, even if stopped
		
		private final function RunFirst():void
		{
			CONFIG::DEBUG
			{
				this._timer = getTimer();
			}
			this.StepFirst(this._context);
			setTimeout(this.RunInit, 0);
		}
		
		private final function RunInit():void
		{
			// The first set of steps we execute we record their "latency"
			// Do 500 iterations at a time
			
			if (!this._isExecuting) return;
			
			const start:int = getTimer();
			const end:int = start + Threaded.runtime;
			var prog:int = 0;

			do
			{
				if (!this.Step(500, this._context))
				{
					this.SetProgress(this._totalProgress);
					setTimeout(this.RunLast, 0);
					return;
				}
				prog += 500;
			}
			while (getTimer() < end);
			const finish:int = getTimer();
			this._latency = prog / (finish-start);
			this._init_iters = uint(Threaded.runtime * this._latency);
			
			this.SetProgress(prog);
			this._timeoutId = setTimeout(this.Run, 0);
		}
		private final function Run():void
		{
			if (!this._isExecuting) return;
			this._timeoutId = 0;
			
			const end:int = getTimer() + Threaded.runtime;
			var prog:int = this._progress;
			var iters:uint = this._init_iters;
			var diff:int;

			while (true)
			{
				if (!this.Step(iters, this._context))
				{
					this.SetProgress(this._totalProgress);
					setTimeout(this.RunLast, 0);
					return;
				}
				prog += iters;
				
				diff = end - getTimer();
				if (diff < 0)
				{
					this.SetProgress(prog);
					this._timeoutId = setTimeout(this.Run, 0);
					return;
				}
				iters = uint(this._latency * diff) + 1;
			}
		}
		private final function RunLast():void
		{
			this.StepLast(this._context);
			CONFIG::DEBUG
			{
				var end:int = getTimer();
				trace(this._name+" thread took "+((end - this._timer)/1000.0)+" secs");
				MonsterDebugger.trace(this, this._name+" thread took "+((end - this._timer)/1000.0)+" secs");
			}
			this._isExecuting = false;
			this.dispatchEvent(new Event(Threaded.FINISHED));
			this._context = {};
		}
	}
}
