﻿/**

Livewire Flash - Core code for running the Livewire algorithm
Copyright (C) 2011  Jeffrey Bush  jeff@coderforlife.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

**/

package Livewire
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;

	public final class WeightCalculator extends Threaded
	{
		private var _settings:WeightCalculatorSettings;
		
		private var _width_raw:uint, _height_raw:uint;
		private var _data_raw:Vector.<uint> = null;
		
		private var _width:uint, _height:uint, _scale:uint;
		private var _data:Vector.<uint>;
		
		private static function WindowSize(m:uint):uint
		{
			switch (m)
			{
			case WeightCalculatorSettings.NoiseReductionMethod_MedianFilter3pxWindow:
			case WeightCalculatorSettings.NoiseReductionMethod_MeanFilter3pxWindow:
			case WeightCalculatorSettings.NoiseReductionMethod_GaussianFilter3pxWindow:
				return 3;
			case WeightCalculatorSettings.NoiseReductionMethod_MedianFilter5pxWindow:
			case WeightCalculatorSettings.NoiseReductionMethod_MeanFilter5pxWindow:
			case WeightCalculatorSettings.NoiseReductionMethod_GaussianFilter5pxWindow:
				return 5;
			case WeightCalculatorSettings.NoiseReductionMethod_None:
			default:
				return 1;
			}
		}
		private static function ScaleBack(x:uint, scale:uint):uint { return (x + scale - 1) / scale; }

		public final function WeightCalculator(w:uint, h:uint, settings:WeightCalculatorSettings)
		{
			super("Weight Calculator");
			this._settings = settings;
			this._scale = 1; //WindowSize(settings.PixelReduction);
			this._width_raw = w;
			this._height_raw = h;
			this._width = ScaleBack(w, this._scale);
			this._height = ScaleBack(h, this._scale);
			super.SetTotalProgress(this._width*this._height);
			this._data = new Vector.<uint>(this._width*this._height, true);
		}
		
		public final function GetWeights():Vector.<uint> { return this._data; }
		
		public final function GetScale():uint { return this._scale; }

		public final function GetOriginalWidth():uint { return this._width_raw; }
		public final function GetOriginalHeight():uint { return this._height_raw; }

		public final function GetReducedWidth():uint { return this._width; }
		public final function GetReducedHeight():uint { return this._height; }

		public final function Start(img:Bitmap):void
		{
			var data:BitmapData = img.bitmapData;
			this._data_raw = data.getVector(data.rect);
			this.start();
		}
		
		protected final override function StepFirst(context:*):void
		{
			context.i = int(0);
		}
		
		protected final override function Step(iters:uint, context:*):Boolean
		{
			for (var iter:uint = 0; iter < iters; ++iter)
			{
				this._data[context.i] = this._data_raw[context.i] & 0xFF;
				if (++context.i == this._width * this._height)
					return false;
			}
			return true;
		}
	}
}
