﻿/**

Livewire Flash - Core code for running the Livewire algorithm
Copyright (C) 2011  Jeffrey Bush  jeff@coderforlife.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

**/

package Livewire
{
	public final class WeightCalculatorSettings
	{
		public static const GrayScaleSettings:WeightCalculatorSettings = new WeightCalculatorSettings();
		public static const ColorSettings:WeightCalculatorSettings = new WeightCalculatorSettings(
			CoalescingMethod_WeightedHSV,
			NoiseReductionMethod_MeanFilter3pxWindow,
			NoiseReductionMethod_None,
			EdgeDetectionMethod_Sobel,
			AccentuationMethod_None,
			false
		);

		public static const CoalescingMethod_RedChannel:uint = 0;
		public static const CoalescingMethod_GreenChannel:uint = 1;
		public static const CoalescingMethod_BlueChannel:uint = 2;
		public static const CoalescingMethod_AvgRGB:uint = 3;
		public static const CoalescingMethod_Luma:uint = 4; // Rec. 709
		public static const CoalescingMethod_Luma601:uint = 5; // Rec. 601
		public static const CoalescingMethod_LumaSMPTE:uint = 6; // SMPTE 240M
		public static const CoalescingMethod_WeightedHSV:uint = 7; // 0.6 * V + 0.3 * H + 0.1 * S
		public static const CoalescingMethod_WeightedHSL:uint = 8; // 0.6 * L + 0.3 * H + 0.1 * S
		public static const CoalescingMethod_WeightedHSI:uint = 9; // 0.6 * I + 0.3 * H + 0.1 * S

		public static const NoiseReductionMethod_None:uint = 0;
		public static const NoiseReductionMethod_MedianFilter3pxWindow:uint = 1;
		public static const NoiseReductionMethod_MedianFilter5pxWindow:uint = 2;
		public static const NoiseReductionMethod_MeanFilter3pxWindow:uint = 3;
		public static const NoiseReductionMethod_MeanFilter5pxWindow:uint = 4;
		public static const NoiseReductionMethod_GaussianFilter3pxWindow:uint = 5;
		public static const NoiseReductionMethod_GaussianFilter5pxWindow:uint = 6;

		public static const EdgeDetectionMethod_None:uint = 0;
		public static const EdgeDetectionMethod_Sobel:uint = 1;
		
		public static const AccentuationMethod_None:uint = 0;
		public static const AccentuationMethod_Sigmoid:uint = 1;
		
		public var Method:uint;
		public var PixelReduction:uint;
		public var NoiseReduction:uint;
		public var EdgeDetection:uint;
		public var Accentuation:uint;
		public var Invert:Boolean;

		public function WeightCalculatorSettings(
				Method:uint = CoalescingMethod_BlueChannel,
				PixelReduction:uint = NoiseReductionMethod_MeanFilter3pxWindow,
				NoiseReduction:uint = NoiseReductionMethod_None,
				EdgeDetection:uint = EdgeDetectionMethod_None,
				Accentuation:uint = AccentuationMethod_Sigmoid,
				Invert:Boolean = false)
		{
			this.Method = Method;
			this.PixelReduction = PixelReduction;
			this.NoiseReduction = NoiseReduction;
			this.EdgeDetection = EdgeDetection;
			this.Accentuation = Accentuation;
			this.Invert = Invert;
		}
	}
}
