﻿/**

Livewire Flash Demo - Demo running the Livewire algorithm
Copyright (C) 2011  Jeffrey Bush  jeff@coderforlife.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

**/

package
{
    import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Graphics;
	import flash.display.Loader;
	import flash.display.LoaderInfo;
	import flash.display.Sprite;
	import flash.display.StageScaleMode;

	import flash.net.URLRequest;

	import flash.geom.Point;

	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFieldType;
	import flash.text.TextLineMetrics;

	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.ProgressEvent;

	import Livewire.*;
	import Livewire.Pools.IntVectors;

	CONFIG::DEBUG
	{
    	import com.demonsters.debugger.MonsterDebugger;
	}
	
    public final class Main extends Sprite
	{
		private var img:Bitmap;
		private var w:uint, h:uint;
		
		private var weights:WeightCalculator;
		private var livewire:LivewireCalculator;
		//private var wrapwire:LivewireCalculator;

		private var prevwires:Sprite = new Sprite(), overlay:Sprite = new Sprite();
		private var points:Vector.<Point> = new Vector.<Point>();
		private var mouse_x:uint = 0, mouse_y:uint = 0;
		private var progressText:TextField = new TextField();

		private var _vectors:IntVectors = new IntVectors(4);

        public final function Main()
		{
			CONFIG::DEBUG
			{
				MonsterDebugger.initialize(this);
			}

			this.stage.scaleMode = StageScaleMode.NO_SCALE;

			this.stage.addEventListener(MouseEvent.MOUSE_MOVE, this.onMouseMove);
			this.stage.addEventListener(MouseEvent.CLICK, this.onClick);
			this.stage.addEventListener(MouseEvent.DOUBLE_CLICK, this.onDblClick);
			
			this.prevwires.graphics.lineStyle(1.5, 0xFFFF00);
			
			this.addChild(prevwires);
			this.addChild(overlay);

			this.progressText.x = 2;
			this.progressText.y = 2;
			this.progressText.autoSize = TextFieldAutoSize.LEFT;
			this.progressText.selectable = false;
			this.progressText.type = TextFieldType.DYNAMIC;
			this.addChild(this.progressText);

			//this.stop();
			
			var loader:Loader = new Loader();
			loader.contentLoaderInfo.addEventListener(Event.COMPLETE, this.ImgLoaded);
			loader.load(new URLRequest("test.png"));
        }
		
		private final function ImgLoaded(evt:Event):void
		{
			trace("Loaded image");
			this.SetImage(Bitmap(LoaderInfo(evt.target).content));
		}
		
		public final function SetImage(img:Bitmap)
		{
			if (this.img != null)
			{
				this.removeChild(this.img);
				this.prevwires.graphics.clear();
				this.overlay.graphics.clear();
				this.points.length = 0;
				this.removeEventsFromThreaded(this.weights);
				this.weights = null;
				this.removeEventsFromThreaded(this.livewire);
				this.livewire = null;
				//this.removeEventsFromThreaded(this.wrapwire);
				//this.wrapwire = null;
			}
			
			this.img = img;
			
			if (img == null)
			{
				this.w = 0;
				this.h = 0;
			}
			else
			{
				this.w = img.width;
				this.h = img.height;
				this.addChildAt(this.img, 0); // must be placed on bottom
					
				this.addEventsToThreaded(this.weights = new WeightCalculator(this.w, this.h, WeightCalculatorSettings.GrayScaleSettings));
				this.addEventsToThreaded(this.livewire = new LivewireCalculator(this.weights));
				
				this.weights.Start(img);
			}
		}

		private final function doUpdate(e:Event):void { this.update(); }
		private final function addEventsToThreaded(t:Threaded):void
		{
			t.addEventListener(Threaded.PROGRESS, this.doUpdate);
			t.addEventListener(Threaded.FINISHED, this.doUpdate);
		}
		private final function removeEventsFromThreaded(t:Threaded):void
		{
			if (t != null)
			{
				t.removeEventListener(Threaded.PROGRESS, this.doUpdate);
				t.removeEventListener(Threaded.FINISHED, this.doUpdate);
			}
		}
		
		private final function drawProgress(g:Graphics, text:String, t:Threaded)
		{
			var percent:Number = Number(t.GetProgress()) / t.GetTotalProgress();
			progressText.text = text + ((int)(percent * 100)) + "%";
			var w:uint = progressText.width, h:uint = progressText.height;
			g.lineStyle(1, 0x000000); g.beginFill(0xFFFFFF); g.drawRect(0, 0, w+2, h+2);     g.endFill();
			g.lineStyle(1, 0x0000FF); g.beginFill(0x0000FF); g.drawRect(1, 1, w*percent, h); g.endFill();
		}
		
		private final function drawTrace(g:Graphics, lw:LivewireCalculator)
		{
			var pts:Vector.<Number> = lw.GetRawTrace(this.mouse_x, this.mouse_y);
			var len:uint = pts.length / 2;
			if (len > 1)
			{
				var cmds:Vector.<int> = this._vectors.Get(len, true);
				cmds[0] = 1; // moveTo
				for (var i:uint = 1; i < len; ++i)
					cmds[i] = 2; // lineTo
				g.drawPath(cmds, pts);
				this._vectors.Return(cmds);
			}
			lw.ReturnRawTrace(pts);
		}
		
		private final function update():void
		{
			var g:Graphics = this.overlay.graphics;
			g.clear();

			if (this.weights == null)
			{
				// Nothing to do
			}
			else if (this.weights.IsExecuting())
			{
				// The weight data is not completely calculated, so we draw a progress message
				this.drawProgress(g, "Calculating weights... ", this.weights);
			}
			else
			{
				// The weights are calculated so we can do normal painting
				// Draw the previous, wrapping, and current livewires
				if (this.points.length > 0)
				{
					//if (this.wrapwire != null)
					//{
					//	g.lineStyle(1.5, 0xFF3F3F, 0.5);
					//	this.drawTrace(g, this.wrapwire);
					//}
					g.lineStyle(1.5, 0xFF0000);
					this.drawTrace(g, this.livewire);
				}
			
				g.lineStyle();
				g.beginFill(0xFFFF00);
				for (var i:uint = 0; i < this.points.length; ++i)
					g.drawCircle(this.points[i].x, this.points[i].y, 4); // draw the seed points
				g.endFill();
				
				g.beginFill(0xFF0000);
				g.drawCircle(this.mouse_x, this.mouse_y, 4); // draw the mouse indicator
				g.endFill();
			
				if (this.livewire.IsExecuting())
				{
					// The livewire data is not completely calculated, so we draw a progress message
					this.drawProgress(g, "Calculating livewire... ", this.livewire);
				}
				else
				{
					progressText.text = "";
				}
			}
		}
		
		private final function getPointInBoundsX(x:uint) { return (x < 0) ? 0 : ((x >= this.w) ? this.w - 1 : x); }
		private final function getPointInBoundsY(y:uint) { return (y < 0) ? 0 : ((y >= this.h) ? this.h - 1 : y); }

		private final function onMouseMove(evnt:MouseEvent):void
		{
			this.mouse_x = this.getPointInBoundsX(this.mouseX);
			this.mouse_y = this.getPointInBoundsY(this.mouseY);
			this.update();
			evnt.updateAfterEvent();
		}
		private final function onClick(evnt:MouseEvent):void { this.mouseClicked(evnt, false); }
		private final function onDblClick(evnt:MouseEvent):void { this.mouseClicked(evnt, true); }
		private final function mouseClicked(evnt:MouseEvent, dbl:Boolean):void 
		{
			if (this.livewire != null)
			{
				var have_points:Boolean = this.points.length > 0;
				this.mouse_x = this.getPointInBoundsX(this.mouseX);
				this.mouse_y = this.getPointInBoundsY(this.mouseY);
				var lw:LivewireCalculator = this.livewire;
				
				//if (dbl && this.wrapwire != null)
				//{
				//	// On double click draw the wrapping livewire
				//	this.drawTrace(this.prevwires.graphics, this.wrapwire);
				//	this.wrapwire = null;
				//}
				//else if (have_points && this.wrapwire == null)
				//{
				//	// Start wrapping from a new point
				//	this.wrapwire = lw;
				//	this.addEventsToThreaded(this.livewire = new LivewireCalculator(this.w, this.h, this.weights.GetWeights()));
				//}
		
				if (have_points)
				{
					// On a single click save the livewire
					this.drawTrace(this.prevwires.graphics, lw);
				}
		
				// Start preparing the next livewire
				this.livewire.Start(this.mouse_x, this.mouse_y);
		
				this.points.push(new Point(this.mouse_x, this.mouse_y));
				this.update();
				evnt.updateAfterEvent();
			}
		}
    }
}
